@@ -4,7 +4,7 @@ import datetime
 import sys
 import warnings
 import time as mod_time
-from redis._compat import (b, izip, imap, iteritems, dictkeys, dictvalues,
+from redis._compat import (b, izip, imap, iteritems, iterkeys, itervalues,
                            basestring, long, nativestr, urlparse, bytes)
 from redis.connection import ConnectionPool, UnixDomainSocketConnection
 from redis.exceptions import (
@@ -104,11 +104,26 @@ def parse_info(response):
 
     for line in response.splitlines():
         if line and not line.startswith('#'):
-            key, value = line.split(':')
+            key, value = line.split(':', 1)
             info[key] = get_value(value)
     return info
 
 
+def parse_sentinel(response, **options):
+    "Parse the result of Redis's SENTINEL command"
+    output = []
+    parse = options['parse']
+
+    if parse == 'SENTINEL_INFO':
+        for sub_list in response:
+            it = iter(sub_list)
+            output.append(dict(izip(it, it)))
+    else:
+        output = response
+
+    return output
+
+
 def pairs_to_dict(response):
     "Create a dict given a list of key/value pairs"
     it = iter(response)
@@ -243,8 +258,9 @@ class StrictRedis(object):
             'PING': lambda r: nativestr(r) == 'PONG',
             'RANDOMKEY': lambda r: r and r or None,
             'SCRIPT': parse_script,
-            'SET': lambda r: r and nativestr(r) == 'OK' or None,
-            'TIME': lambda x: (int(x[0]), int(x[1]))
+            'SET': lambda r: r and nativestr(r) == 'OK',
+            'TIME': lambda x: (int(x[0]), int(x[1])),
+            'SENTINEL': parse_sentinel
         }
     )
 
@@ -426,26 +442,18 @@ class StrictRedis(object):
         "Set config item ``name`` with ``value``"
         return self.execute_command('CONFIG', 'SET', name, value, parse='SET')
 
+    def config_resetstat(self):
+        "Reset runtime statistics"
+        return self.execute_command('CONFIG', 'RESETSTAT', parse='RESETSTAT')
+
     def dbsize(self):
         "Returns the number of keys in the current database"
         return self.execute_command('DBSIZE')
 
-    def time(self):
-        """
-        Returns the server time as a 2-item tuple of ints:
-        (seconds since epoch, microseconds into this second).
-        """
-        return self.execute_command('TIME')
-
     def debug_object(self, key):
         "Returns version specific metainformation about a give key"
         return self.execute_command('DEBUG', 'OBJECT', key)
 
-    def delete(self, *names):
-        "Delete one or more keys specified by ``names``"
-        return self.execute_command('DEL', *names)
-    __delitem__ = delete
-
     def echo(self, value):
         "Echo the string back from the server"
         return self.execute_command('ECHO', value)
@@ -495,6 +503,14 @@ class StrictRedis(object):
         """
         return self.execute_command('SAVE')
 
+    def sentinel(self, *args):
+        "Redis Sentinel's SENTINEL command"
+        if args[0] in ['masters', 'slaves', 'sentinels']:
+            parse = 'SENTINEL_INFO'
+        else:
+            parse = 'SENTINEL'
+        return self.execute_command('SENTINEL', *args, **{'parse': parse})
+
     def shutdown(self):
         "Shutdown the server"
         try:
@@ -514,6 +530,13 @@ class StrictRedis(object):
             return self.execute_command("SLAVEOF", "NO", "ONE")
         return self.execute_command("SLAVEOF", host, port)
 
+    def time(self):
+        """
+        Returns the server time as a 2-item tuple of ints:
+        (seconds since epoch, microseconds into this second).
+        """
+        return self.execute_command('TIME')
+
     #### BASIC KEY COMMANDS ####
     def append(self, key, value):
         """
@@ -523,13 +546,6 @@ class StrictRedis(object):
         """
         return self.execute_command('APPEND', key, value)
 
-    def getrange(self, key, start, end):
-        """
-        Returns the substring of the string value stored at ``key``,
-        determined by the offsets ``start`` and ``end`` (both are inclusive)
-        """
-        return self.execute_command('GETRANGE', key, start, end)
-
     def bitcount(self, key, start=None, end=None):
         """
         Returns the count of set bits in the value of ``key``.  Optional
@@ -558,6 +574,18 @@ class StrictRedis(object):
         """
         return self.execute_command('DECRBY', name, amount)
 
+    def delete(self, *names):
+        "Delete one or more keys specified by ``names``"
+        return self.execute_command('DEL', *names)
+    __delitem__ = delete
+
+    def dump(self, name):
+        """
+        Return a serialized version of the value stored at the specified key.
+        If key does not exist a nil bulk reply is returned.
+        """
+        return self.execute_command('DUMP', name)
+
     def exists(self, name):
         "Returns a boolean indicating whether key ``name`` exists"
         return self.execute_command('EXISTS', name)
@@ -601,6 +629,13 @@ class StrictRedis(object):
         "Returns a boolean indicating the value of ``offset`` in ``name``"
         return self.execute_command('GETBIT', name, offset)
 
+    def getrange(self, key, start, end):
+        """
+        Returns the substring of the string value stored at ``key``,
+        determined by the offsets ``start`` and ``end`` (both are inclusive)
+        """
+        return self.execute_command('GETRANGE', key, start, end)
+
     def getset(self, name, value):
         """
         Set the value at key ``name`` to ``value`` if key doesn't exist
@@ -643,20 +678,33 @@ class StrictRedis(object):
         args = list_or_args(keys, args)
         return self.execute_command('MGET', *args)
 
-    def mset(self, mapping):
-        "Sets each key in the ``mapping`` dict to its corresponding value"
+    def mset(self, *args, **kwargs):
+        """
+        Sets key/values based on a mapping. Mapping can be supplied as a single
+        dictionary argument or as kwargs.
+        """
+        if args:
+            if len(args) != 1 or not isinstance(args[0], dict):
+                raise RedisError('MSET requires **kwargs or a single dict arg')
+            kwargs.update(args[0])
         items = []
-        for pair in iteritems(mapping):
+        for pair in iteritems(kwargs):
             items.extend(pair)
         return self.execute_command('MSET', *items)
 
-    def msetnx(self, mapping):
+    def msetnx(self, *args, **kwargs):
         """
-        Sets each key in the ``mapping`` dict to its corresponding value if
-        none of the keys are already set
+        Sets key/values based on a mapping if none of the keys are already set.
+        Mapping can be supplied as a single dictionary argument or as kwargs.
+        Returns a boolean indicating if the operation was successful.
         """
+        if args:
+            if len(args) != 1 or not isinstance(args[0], dict):
+                raise RedisError('MSETNX requires **kwargs or a single '
+                                 'dict arg')
+            kwargs.update(args[0])
         items = []
-        for pair in iteritems(mapping):
+        for pair in iteritems(kwargs):
             items.extend(pair)
         return self.execute_command('MSETNX', *items)
 
@@ -719,6 +767,13 @@ class StrictRedis(object):
         "Rename key ``src`` to ``dst`` if ``dst`` doesn't already exist"
         return self.execute_command('RENAMENX', src, dst)
 
+    def restore(self, name, ttl, value):
+        """
+        Create a key using the provided serialized value, previously obtained
+        using DUMP.
+        """
+        return self.execute_command('RESTORE', name, ttl, value)
+
     def set(self, name, value, ex=None, px=None, nx=False, xx=False):
         """
         Set the value at key ``name`` to ``value``
@@ -1138,6 +1193,10 @@ class StrictRedis(object):
         return self.execute_command('ZCARD', name)
 
     def zcount(self, name, min, max):
+        """
+        Returns the number of elements in the sorted set at key ``name`` with
+        a score between ``min`` and ``max``.
+        """
         return self.execute_command('ZCOUNT', name, min, max)
 
     def zincrby(self, name, value, amount=1):
@@ -1298,7 +1357,7 @@ class StrictRedis(object):
     def _zaggregate(self, command, dest, keys, aggregate=None):
         pieces = [command, dest, len(keys)]
         if isinstance(keys, dict):
-            keys, weights = dictkeys(keys), dictvalues(keys)
+            keys, weights = iterkeys(keys), itervalues(keys)
         else:
             weights = None
         pieces.extend(keys)
@@ -1808,10 +1867,18 @@ class BasePipeline(object):
             starmap(connection.pack_command,
                     [args for args, options in cmds]))
         connection.send_packed_command(all_cmds)
-        # parse off the response for MULTI
-        self.parse_response(connection, '_')
-        # and all the other commands
         errors = []
+
+        # parse off the response for MULTI
+        # NOTE: we need to handle ResponseErrors here and continue
+        # so that we read all the additional command messages from
+        # the socket
+        try:
+            self.parse_response(connection, '_')
+        except ResponseError:
+            errors.append((0, sys.exc_info()[1]))
+
+        # and all the other commands
         for i, _ in enumerate(commands):
             try:
                 self.parse_response(connection, '_')
@@ -1969,7 +2036,8 @@ class Script(object):
 
     def __call__(self, keys=[], args=[], client=None):
         "Execute the script, passing any required ``args``"
-        client = client or self.registered_client
+        if client is None:
+            client = self.registered_client
         args = tuple(keys) + tuple(args)
         # make sure the Redis server knows about the script
         if isinstance(client, BasePipeline):
@@ -2041,7 +2109,7 @@ class Lock(object):
         sleep = self.sleep
         timeout = self.timeout
         while 1:
-            unixtime = int(mod_time.time())
+            unixtime = mod_time.time()
             if timeout:
                 timeout_at = unixtime + timeout
             else:
