@@ -3,24 +3,23 @@ import os
 import socket
 import sys
 
+
 from redis._compat import (b, xrange, imap, byte_to_chr, unicode, bytes, long,
                            BytesIO, nativestr, basestring,
                            LifoQueue, Empty, Full)
 from redis.exceptions import (
     RedisError,
     ConnectionError,
+    BusyLoadingError,
     ResponseError,
     InvalidResponse,
     AuthenticationError,
     NoScriptError,
     ExecAbortError,
 )
-
-try:
+from redis.utils import HIREDIS_AVAILABLE
+if HIREDIS_AVAILABLE:
     import hiredis
-    hiredis_available = True
-except ImportError:
-    hiredis_available = False
 
 
 SYM_STAR = b('*')
@@ -37,7 +36,7 @@ class PythonParser(object):
     EXCEPTION_CLASSES = {
         'ERR': ResponseError,
         'EXECABORT': ExecAbortError,
-        'LOADING': ConnectionError,
+        'LOADING': BusyLoadingError,
         'NOSCRIPT': NoScriptError,
     }
 
@@ -115,9 +114,16 @@ class PythonParser(object):
         # server returned an error
         if byte == '-':
             response = nativestr(response)
-            # *return*, not raise the exception class. if it is meant to be
-            # raised, it will be at a higher level.
-            return self.parse_error(response)
+            error = self.parse_error(response)
+            # if the error is a ConnectionError, raise immediately so the user
+            # is notified
+            if isinstance(error, ConnectionError):
+                raise error
+            # otherwise, we're dealing with a ResponseError that might belong
+            # inside a pipeline response. the connection's read_response()
+            # and/or the pipeline's execute() will raise this error if
+            # necessary, so just return the exception instance here.
+            return error
         # single value
         elif byte == '+':
             pass
@@ -144,7 +150,7 @@ class PythonParser(object):
 class HiredisParser(object):
     "Parser class for connections using Hiredis"
     def __init__(self):
-        if not hiredis_available:
+        if not HIREDIS_AVAILABLE:
             raise RedisError("Hiredis is not installed")
 
     def __del__(self):
@@ -188,7 +194,7 @@ class HiredisParser(object):
             response = self._reader.gets()
         return response
 
-if hiredis_available:
+if HIREDIS_AVAILABLE:
     DefaultParser = HiredisParser
 else:
     DefaultParser = PythonParser
@@ -270,6 +276,7 @@ class Connection(object):
         if self._sock is None:
             return
         try:
+            self._sock.shutdown(socket.SHUT_RDWR)
             self._sock.close()
         except socket.error:
             pass
@@ -290,7 +297,7 @@ class Connection(object):
                 _errno, errmsg = e.args
             raise ConnectionError("Error %s while writing to socket. %s." %
                                   (_errno, errmsg))
-        except Exception:
+        except:
             self.disconnect()
             raise
 
@@ -302,7 +309,7 @@ class Connection(object):
         "Read the response from a previously sent command"
         try:
             response = self._parser.read_response()
-        except Exception:
+        except:
             self.disconnect()
             raise
         if isinstance(response, ResponseError):
@@ -323,14 +330,17 @@ class Connection(object):
 
     def pack_command(self, *args):
         "Pack a series of arguments into a value Redis command"
-        output = SYM_STAR + b(str(len(args))) + SYM_CRLF
+        output = BytesIO()
+        output.write(SYM_STAR)
+        output.write(b(str(len(args))))
+        output.write(SYM_CRLF)
         for enc_value in imap(self.encode, args):
-            output += SYM_DOLLAR
-            output += b(str(len(enc_value)))
-            output += SYM_CRLF
-            output += enc_value
-            output += SYM_CRLF
-        return output
+            output.write(SYM_DOLLAR)
+            output.write(b(str(len(enc_value))))
+            output.write(SYM_CRLF)
+            output.write(enc_value)
+            output.write(SYM_CRLF)
+        return output.getvalue()
 
 
 class UnixDomainSocketConnection(Connection):
